
#ifndef BLAHTECH_SUP_DEM_CONTROLLER_MQH
#define BLAHTECH_SUP_DEM_CONTROLLER_MQH

#property description "Blahtech Supply Demand Controller"
#property link        "http://www.blahtech.co.uk"
#property strict

#include "BlahtechSupDemLevel.mqh"
#include "BlahtechSupDemCustomArgs.mqh"

class SupDemControlAdvanced
{
public:
   bool              ForceFullUpdate;
   
   bool              NewBar;
   int               OverallTrend;
   int               Trend1;
   int               Engulfing1;
   double            QTop1;
   double            QBottom1;
   int               SupDemHint1;
   
   int               Trend2;
   int               Engulfing2;
   double            QTop2;
   double            QBottom2;
   int               SupDemHint2;
   
   int               Trend3;
   int               Engulfing3;
   double            QTop3;
   double            QBottom3;
   
   double            QHi1;
   double            QHi2;
   double            QHi3;
   double            QLow1;
   double            QLow2;
   double            QLow3;
   
   SupDemLevel*      SupLevels1[];
   SupDemLevel*      DemLevels1[];
   SupDemLevel*      SupLevels2[];
   SupDemLevel*      DemLevels2[];
   SupDemLevel*      SupLevels3[];
   SupDemLevel*      DemLevels3[];
   
   SupDemLevelIdx*   LevelIndex1;
   SupDemLevelIdx*   LevelIndex2;
   SupDemLevelIdx*   LevelIndex3;
   
private:
   SupDemCustomArgs* CustomArgs;
   LogLevelEnum      LogLevel;
   SdEaValidateEnum  ValidateIncremental;
   SdEaRetouchMode   UpdateRetouches;
   int               IndicatorHandle;
   bool              ValidateActive;
   uint              TickCount;
   datetime          PrevTime1_0;
   datetime          PrevTime2_0;
   datetime          PrevTime3_0;
   
public:
   ~SupDemControlAdvanced();
   int             Init(SupDemCustomArgs* customArgs, SdEaRetouchMode updateRetouches,  LogLevelEnum logLevel, SdEaValidateEnum  validateInc, bool forceFullUpdate);
   SdEaUpdateEnum  Populate(datetime time0);
   
private:
   double          iCustomSd(SdBufferIndexEnum index, int shift = 0);
   
   void            PopulateLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx);
   void            PopulateLevelsSupplies(SupDemLevel* &levels[], const SupDemLevelIdx& levelIdx, int barNum = 0);
   void            PopulateLevelsDemands(SupDemLevel* &levels[], const SupDemLevelIdx& levelIdx, int barNum = 0);
   void            IncrementalLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll = false, bool isNewBar = false);
   void            IncrementalSupplies(SupDemLevel* &supLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll);
   void            IncrementalDemands(SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll);
   bool            ValidateLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, string prefix, bool validateAll = false);
   
   SupDemLevel*    CreateNewSupply(const SupDemLevelIdx& levelIdx, int barNum);
   SupDemLevel*    CreateNewDemand(const SupDemLevelIdx& levelIdx, int barNum);
   
   void            UpdateRetouchesSupply(SupDemLevel& sup);
   void            UpdateRetouchesDemand(SupDemLevel& dem);
};


void
SupDemControlAdvanced::~SupDemControlAdvanced()
{
   ArrayDelete(SupLevels1);
   ArrayDelete(DemLevels1);
   ArrayDelete(SupLevels2);
   ArrayDelete(DemLevels2);
   ArrayDelete(SupLevels3);
   ArrayDelete(DemLevels3);
   DeleteCheck(LevelIndex1);
   DeleteCheck(LevelIndex2);
   DeleteCheck(LevelIndex3);
#ifndef __MQL4__
   if (IndicatorHandle > 0)
   {
      IndicatorRelease(IndicatorHandle);
   }
#endif
}


double
SupDemControlAdvanced::iCustomSd(SdBufferIndexEnum index, int shift)
{
#ifdef __MQL4__
   return 
      iCustom(
         CustomArgs.StrSymbol, CustomArgs.Timeframe, CustomArgs.IndicatorPath,
         CustomArgs.ExpertMode, CustomArgs.MinBarsLevel, CustomArgs.MaxBars, CustomArgs.ForceRecalcBars, CustomArgs.ParameterValues,
         index, shift);
#else
   double arr[1];
   int    retIdx = CopyBuffer(IndicatorHandle, index, shift, 1, arr);
   return retIdx > 0 ? arr[0] : EMPTY_VALUE;
#endif
}


int
SupDemControlAdvanced::Init(SupDemCustomArgs* customArgs, SdEaRetouchMode updateRetouches,  LogLevelEnum logLevel, SdEaValidateEnum  validateInc, bool forceFullUpdate)
{
   int ret = INIT_FAILED;
   
   if (LogLevel >= LogInfo) { Print("SupDemControlAdvanced::Init - Start"); }
   
   CustomArgs = customArgs;
   UpdateRetouches = updateRetouches;
   LogLevel = logLevel;
   ValidateIncremental = validateInc;
   ValidateActive = ValidateIncremental != SD_EA_VAL_NONE;
   ForceFullUpdate = forceFullUpdate;
   
   LevelIndex1 = new SupDemLevelIdx(1, MappedTimeframeToTimeframe(TIMEFRAME_MAP1, CustomArgs.Timeframe1), SD_SUPBARNUM,  SD_SUPLOW,  SD_DEMBARNUM,  SD_DEMHIGH);
   LevelIndex2 = new SupDemLevelIdx(2, MappedTimeframeToTimeframe(TIMEFRAME_MAP2, CustomArgs.Timeframe2), SD_SUPBARNUM2, SD_SUPLOW2, SD_DEMBARNUM2, SD_DEMHIGH2);
   LevelIndex3 = new SupDemLevelIdx(3, MappedTimeframeToTimeframe(TIMEFRAME_MAP3, CustomArgs.Timeframe3), SD_SUPBARNUM3, SD_SUPLOW3, SD_DEMBARNUM3, SD_DEMHIGH3);
   
   CustomArgs.ParameterValues += "Timeframe1=" + IntegerToString(LevelIndex1.Timeframe) + ";";
   CustomArgs.ParameterValues += "Timeframe2=" + IntegerToString(LevelIndex2.Timeframe) + ";";
   CustomArgs.ParameterValues += "Timeframe3=" + IntegerToString(LevelIndex3.Timeframe) + ";";
   
   Print("SupDemControlAdvanced::Init - ParameterValues=");
   Print(CustomArgs.ParameterValues);

#ifdef __MQL4__
   ret = INIT_SUCCEEDED;
#else
   IndicatorHandle =
      iCustom(
         customArgs.StrSymbol, customArgs.Timeframe, customArgs.IndicatorPath,
         customArgs.ExpertMode, customArgs.MinBarsLevel, customArgs.MaxBars, customArgs.ForceRecalcBars, customArgs.ParameterValues);
         
   ret = IndicatorHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
#endif
   if (LogLevel >= LogInfo) { Print("SupDemControlAdvanced::Init - IndicatorPath=", CustomArgs.IndicatorPath); }
   return ret;
}


SdEaUpdateEnum
SupDemControlAdvanced::Populate(datetime time0)
{
   SdEaUpdateEnum retChanged = (SdEaUpdateEnum)iCustomSd(SD_CHANGED); // Only do work when values have changed

   TickCount++;
   NewBar = time0 != PrevTime1_0;
   
   //if (retChanged == SD_EA_UPDATE_NONE) // Uncomment this block to simulate incremental updates every tick. This allows us to run the MT4 profliler for incremental changes
   //{
   //   retChanged = SD_EA_UPDATE_INCREMENTAL;
   //}
   if (retChanged > SD_EA_UPDATE_NONE || NewBar)
   {
      if (ForceFullUpdate)
      {
         retChanged = SD_EA_UPDATE_FULL;
      }
      if ((retChanged > SD_EA_UPDATE_NONE && LogLevel >= LogDebug) || LogLevel >= LogAll) { Print("SupDemControlAdvanced::Populate TickCount=", TickCount, " retChanged=", retChanged); }

      // ######################################################################################################
      // ##  For optimal performance - Comment out iCustom calls below for any variables that are not required  
      // ######################################################################################################
      
      OverallTrend = (int)iCustomSd(SD_OVERALLTREND);  // Overall Trend
      Trend1       = (int)iCustomSd(SD_TREND1);        // Timeframe 1
      Engulfing1   = (int)iCustomSd(SD_ENGULFING1);
      QTop1        = iCustomSd(SD_QTOP1);
      QBottom1     = iCustomSd(SD_QBOTTOM1);
      SupDemHint1  = (int)iCustomSd(SD_SUPDEMHINT1);
      Trend2       = (int)iCustomSd(SD_TREND2);        // Timeframe 2
      Engulfing2   = (int)iCustomSd(SD_ENGULFING2);
      QTop2        = iCustomSd(SD_QTOP2);
      QBottom2     = iCustomSd(SD_QBOTTOM2);
      SupDemHint2  = (int)iCustomSd(SD_SUPDEMHINT2);
      Trend3       = (int)iCustomSd(SD_TREND3);        // Timeframe 3
      Engulfing3   = (int)iCustomSd(SD_ENGULFING3);
      QTop3        = iCustomSd(SD_QTOP3);
      QBottom3     = iCustomSd(SD_QBOTTOM3);
      
      // Calculate the QHi and QLow values
      QHi1         = QTop1 * 0.75 + QBottom1 * 0.25;
      QHi2         = QTop2 * 0.75 + QBottom2 * 0.25;
      QHi3         = QTop3 * 0.75 + QBottom3 * 0.25;
      QLow1        = QTop1 * 0.25 + QBottom1 * 0.75;
      QLow2        = QTop2 * 0.25 + QBottom2 * 0.75;
      QLow3        = QTop3 * 0.25 + QBottom3 * 0.75;
      
      if (retChanged == SD_EA_UPDATE_FULL)
      {
         PopulateLevels(SupLevels1, DemLevels1, LevelIndex1);
         PopulateLevels(SupLevels2, DemLevels2, LevelIndex2);
         PopulateLevels(SupLevels3, DemLevels3, LevelIndex3);
         PrevTime2_0 = LevelIndex2.IsValid() ? iTime(NULL, LevelIndex2.Timeframe, 0) : 0;
         PrevTime3_0 = LevelIndex3.IsValid() ? iTime(NULL, LevelIndex3.Timeframe, 0) : 0;
      }
      else
      {
         bool updateRetouchesAll = UpdateRetouches == SD_EA_RETOUCH_ALL;
         
         if (NewBar)
         {
            IncrementalLevels(SupLevels1, DemLevels1, LevelIndex1, updateRetouchesAll, NewBar); // Populate incremental levels
            
            if (ValidateActive && ! ValidateLevels(SupLevels1, DemLevels1, LevelIndex1, "1", updateRetouchesAll))
            {
               PopulateLevels(SupLevels1, DemLevels1, LevelIndex1);
               if (ValidateIncremental == SD_EA_VAL_FIRST) { ValidateActive = false; }
            }
            // Timeframe 2
            if (LevelIndex2.IsValid() && LevelIndex2.Timeframe != LevelIndex1.Timeframe)
            {
               datetime time2_0 = iTime(NULL, LevelIndex2.Timeframe, 0);
               bool     newBar2 = PrevTime2_0 != 0 && PrevTime2_0 != time2_0;
               
               //if (newBar2 && time2_0 > D'2016.12.11 22:54' && time2_0 <  D'2016.12.11 23:10') { Print("newBar2 stPrevTime2_0=", stPrevTime2_0, "  time2_0=", time2_0); }
               IncrementalLevels(SupLevels2, DemLevels2, LevelIndex2, updateRetouchesAll, newBar2); // Populate incremental levels
               
               if (ValidateActive && !ValidateLevels(SupLevels2, DemLevels2, LevelIndex2, "2", updateRetouchesAll))
               {
                  PopulateLevels(SupLevels2, DemLevels2, LevelIndex2);
                  if (ValidateIncremental == SD_EA_VAL_FIRST) { ValidateActive = false; }
               }
               if (PrevTime2_0 != time2_0)
               {
                  PrevTime2_0 = time2_0;
               }
            }
            // Timeframe 3
            if (LevelIndex3.IsValid() && LevelIndex3.Timeframe != LevelIndex1.Timeframe)
            {
               datetime time3_0 = iTime(NULL, LevelIndex3.Timeframe, 0);
               bool     newBar3 = PrevTime3_0 != 0 && PrevTime3_0 != time3_0;
               
               IncrementalLevels(SupLevels3, DemLevels3, LevelIndex3, updateRetouchesAll, newBar3); // Populate incremental levels
               
               if (ValidateActive && !ValidateLevels(SupLevels3, DemLevels3, LevelIndex3, "3", updateRetouchesAll))
               {
                  PopulateLevels(SupLevels3, DemLevels3, LevelIndex3);
                  if (ValidateIncremental == SD_EA_VAL_FIRST) { ValidateActive = false; }
               }
               if (PrevTime3_0 != time3_0)
               {
                  PrevTime3_0 = time3_0;
               }
            }
         }
         else
         {
            IncrementalLevels(SupLevels1, DemLevels1, LevelIndex1);  // Populate incremental levels
            IncrementalLevels(SupLevels2, DemLevels2, LevelIndex2);  // Populate incremental levels
            IncrementalLevels(SupLevels3, DemLevels3, LevelIndex3);  // Populate incremental levels
         }
         //Comment(" Sup  ", ArraySize(SupLevels1), "  ( ", ArrayBarNumToString(SupLevels1), " )    Dem  ", ArraySize(DemLevels1), "  ( ", ArrayBarNumToString(DemLevels1), " )");
      }
      PrevTime1_0 = time0;
   }
   return retChanged;
}


void
SupDemControlAdvanced::PopulateLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx)
{
   if (levelIdx.IsValid())
   {
      ArrayDelete(supLevels);
      ArrayDelete(demLevels);
      PopulateLevelsSupplies(supLevels, levelIdx);
      PopulateLevelsDemands(demLevels, levelIdx);
      if (LogLevel >= LogInfo) { Print(TimeframeToString(levelIdx.Timeframe), " PopulateLevels NumSups=", ArraySize(supLevels), " NumDems=", ArraySize(demLevels)); }
   }
}


void
SupDemControlAdvanced::PopulateLevelsSupplies(SupDemLevel* &levels[], const SupDemLevelIdx& levelIdx, int barNum)
{
   int barNumOffset = (int)iCustomSd(levelIdx.SupBarNum, barNum);
   
   while (barNumOffset > 0 && barNumOffset != EMPTY_VALUE)
   {
      barNum += barNumOffset;
      ArrayAppend(levels, CreateNewSupply(levelIdx, barNum));
      barNumOffset = (int)iCustomSd(levelIdx.SupBarNum, barNum);
   }
   if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " PopulateLevelsSupplies NumSups=", ArraySize(levels)); }
}


void
SupDemControlAdvanced::PopulateLevelsDemands(SupDemLevel* &levels[], const SupDemLevelIdx& levelIdx, int barNum)
{
   int barNumOffset = (int)iCustomSd(levelIdx.DemBarNum, barNum);
   //int targetDems = LogLevel >= LogError && barNum == 0 ? (int)iCustomSd(levelIdx.DemBarNum, 1) : EMPTY_VALUE;
   //string barNumStrs = "";
   
   while (barNumOffset > 0 && barNumOffset != EMPTY_VALUE)
   {
      //barNumStrs += (string)barNum + ":" + (string)barNumOffset + " ";
      barNum += barNumOffset;
      ArrayAppend(levels, CreateNewDemand(levelIdx, barNum));
      barNumOffset = (int)iCustomSd(levelIdx.DemBarNum, barNum);
   }
   //if (targetDems != EMPTY_VALUE && targetDems != ArraySize(levels)) { Print(TimeframeToString(levelIdx.Timeframe), " PopulateLevelsDemands targetDems=", targetDems, "  NumDems=", ArraySize(levels), "   ", barNumStrs); }
   if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " PopulateLevelsDemands NumDems=", ArraySize(levels)); }
}


void
SupDemControlAdvanced::IncrementalLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll = false, bool isNewBar = false)
{
   if (levelIdx.IsValid())
   {
      if (isNewBar)
      {
         int numSups = ArraySize(supLevels);
         int numDems = ArraySize(demLevels);
         
         for(int i = 0; i < numSups; i++)
         {
            supLevels[i].IncrementBarNums();
         }
         for(int i = 0; i < numDems; i++)
         {
            demLevels[i].IncrementBarNums();;
         }
      }
      IncrementalSupplies(supLevels, levelIdx, updateRetouchesAll);
      IncrementalDemands(demLevels, levelIdx, updateRetouchesAll);
   }
}


void
SupDemControlAdvanced::IncrementalSupplies(SupDemLevel* &supLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll)
{
   int numSups = ArraySize(supLevels);
   int bufferSupBarNum = (int)iCustomSd(levelIdx.SupBarNum, 0); // First target bar num
   
   if (bufferSupBarNum > 0 && bufferSupBarNum != EMPTY_VALUE)
   {
      if (numSups < 1)
      {
         PopulateLevelsSupplies(supLevels, levelIdx);
      }
      else
      {
         int newBarNum = 0;
         int startBarNum = supLevels[0].BarNum;  // Get existing start bar num
         int targetNumSups = (int)iCustomSd(levelIdx.SupBarNum, 1); // Target num supplies
         
         if (bufferSupBarNum < startBarNum) // New level detected
         {
            newBarNum = bufferSupBarNum;
            int barNumOffset = (int)iCustomSd(levelIdx.SupBarNum, bufferSupBarNum); // Get the next BarNum offset from the buffer
            
            if (barNumOffset > 0 && barNumOffset != EMPTY_VALUE)
            {
               bufferSupBarNum += barNumOffset; // Set the next bar
               if (LogLevel >= LogInfo && bufferSupBarNum > startBarNum) { Print(TimeframeToString(levelIdx.Timeframe), " Adding new Sup BarNum=", newBarNum, " and removing old Sup BarNum=", startBarNum); }
            }
            else
            {
               bufferSupBarNum = -1;  // No next bar, so delete the old list
               ArrayDelete(supLevels);
            }
         }
         if (bufferSupBarNum > startBarNum) // New supplies start after existing list, so delete supplies from start of list
         {
            int numToDelete = 1;
            
            while (numToDelete < numSups && bufferSupBarNum > supLevels[numToDelete].BarNum)
            {
               numToDelete++;
            }
            ArrayDeleteFromStart(supLevels, numToDelete);
            if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " Sup-", numToDelete, " => ", ArraySize(supLevels)); }
         }
         if (newBarNum > 0) // Add the new supply detected above
         {
            ArrayPrepend(supLevels, CreateNewSupply(levelIdx, newBarNum));
            if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " Sup+1 => ", ArraySize(supLevels)); }
         }
         else if (levelIdx.PopulateScoresAndRetouches() && ArraySize(supLevels) > 0) // Update score/retouch data if required
         {
            SupDemLevel* firstSup = supLevels[0];
            
            firstSup.Score = (int)iCustomSd(SD_SUPSCORE, 0);
            firstSup.Retouch = (int)iCustomSd(SD_SUPRETOUCH, 0);
            firstSup.IsStrong = (bool)iCustomSd(SD_SUPSTRONG, 0);
         }
         int newNumSups = ArraySize(supLevels);
         
         if (targetNumSups >= 0 && targetNumSups != EMPTY_VALUE)
         {
            if (targetNumSups < newNumSups) // Delete old supplies from end of list
            {
               //Print("ArrayDeleteFromEnd targetNumSups=", targetNumSups);
               ArrayDeleteFromEnd(supLevels, newNumSups - targetNumSups);
               newNumSups = targetNumSups; // Update newNumSups
            }
            else if (targetNumSups > newNumSups) // Add new supplies to end of list
            {
               PopulateLevelsSupplies(supLevels, levelIdx, newNumSups > 0 ? supLevels[newNumSups-1].BarNum : 0);
               // leave newNumSups at old value because we don't need to update the new suppplies score/retouch data below
            }
         }
         if (updateRetouchesAll && levelIdx.PopulateScoresAndRetouches()) // Update score/retouch data if required
         {
            for(int i = 1; i < newNumSups; i++)
            {
               UpdateRetouchesSupply(supLevels[i]);
            }
         }
      }
   }
   else if (numSups > 0)
   {
      ArrayDelete(supLevels);
   }
}


void
SupDemControlAdvanced::IncrementalDemands(SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, bool updateRetouchesAll)
{
   int numDems = ArraySize(demLevels);
   int bufferDemBarNum = (int)iCustomSd(levelIdx.DemBarNum, 0); // First target bar num
   
   if (bufferDemBarNum > 0 && bufferDemBarNum != EMPTY_VALUE)
   {
      if (numDems < 1)
      {
         PopulateLevelsDemands(demLevels, levelIdx);
      }
      else
      {
         int newBarNum = 0;
         int startBarNum = demLevels[0].BarNum; // Get existing start bar num
         int targetNumDems = (int)iCustomSd(levelIdx.DemBarNum, 1); // Target num demands
         
         if (bufferDemBarNum < startBarNum) // New level detected
         {
            newBarNum = bufferDemBarNum;  
            int barNumOffset = (int)iCustomSd(levelIdx.DemBarNum, bufferDemBarNum); // Get the next BarNum offset from the buffer
            
            if (barNumOffset > 0 && barNumOffset != EMPTY_VALUE)
            {
               bufferDemBarNum += barNumOffset;  // Setup the next bar
               if (LogLevel >= LogInfo && bufferDemBarNum > startBarNum) { Print(TimeframeToString(levelIdx.Timeframe), " Adding new Dem BarNum=", newBarNum, " and removing old Dem BarNum=", startBarNum); }
            }
            else
            {
               bufferDemBarNum = -1;  // No next bar, so delete the old list
               ArrayDelete(demLevels);
            }
         }
         if (bufferDemBarNum > startBarNum) // New demands start after existing list, so delete demands from start of list
         {
            int numToDelete = 1;
            
            while (numToDelete < numDems && bufferDemBarNum > demLevels[numToDelete].BarNum)
            {
               numToDelete++;
            }
            ArrayDeleteFromStart(demLevels, numToDelete);
            if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " Dem-", numToDelete, " => ", ArraySize(demLevels)); }
         }
         if (newBarNum > 0) // Add the new demand detected above
         {
            ArrayPrepend(demLevels, CreateNewDemand(levelIdx, newBarNum));
            if (LogLevel >= LogDebug) { Print(TimeframeToString(levelIdx.Timeframe), " Dem+1 => ", ArraySize(demLevels)); }
         }
         else if (levelIdx.PopulateScoresAndRetouches() && ArraySize(demLevels) > 0) // Update score/retouch data if required
         {
            SupDemLevel* firstDem = demLevels[0];
            
            firstDem.Score = (int)iCustomSd(SD_DEMSCORE, 0);
            firstDem.Retouch = (int)iCustomSd(SD_DEMRETOUCH, 0);
            firstDem.IsStrong = (bool)iCustomSd(SD_DEMSTRONG, 0);
         }
         int newNumDems = ArraySize(demLevels);
         
         //Print("IncrementalDemands  newNumDems=", newNumDems, "  targetNumDems=", targetNumDems);
         
         if (targetNumDems >= 0 && targetNumDems != EMPTY_VALUE)
         {
            if (targetNumDems < newNumDems) // Delete old demands from end of list
            {
               //Print("ArrayDeleteFromEnd newNumDems=", newNumDems, "  targetNumDems=", targetNumDems);
               ArrayDeleteFromEnd(demLevels, newNumDems - targetNumDems);
               newNumDems = targetNumDems; // Update newNumDems
            }
            else if (targetNumDems > newNumDems) // Add new demands to end of list
            {
               PopulateLevelsDemands(demLevels, levelIdx, newNumDems > 0 ? demLevels[newNumDems-1].BarNum : 0);
               // leave numSups at old value because we don't need to update the new demands score/retouch data below
            }
         }
         if (updateRetouchesAll && levelIdx.PopulateScoresAndRetouches()) // Update score/retouch data if required
         {
            for(int i = 1; i < newNumDems; i++)
            {
               UpdateRetouchesDemand(demLevels[i]);
            }
         }
      }
   }
   else if (numDems > 0)
   {
      ArrayDelete(demLevels);
   }
}


bool
SupDemControlAdvanced::ValidateLevels(SupDemLevel* &supLevels[], SupDemLevel* &demLevels[], const SupDemLevelIdx& levelIdx, string prefix, bool validateAll = false)
{
   bool         ret = true;
   SupDemLevel* validateSups[];
   SupDemLevel* validateDems[];
   
   PopulateLevelsSupplies(validateSups, levelIdx);
   PopulateLevelsDemands(validateDems, levelIdx);
   
   if (!SupDemLevel::CompareLevels("Sups", supLevels, validateSups, !validateAll, LogLevel))
   {
      if (LogLevel >= LogError) { Print(TimeframeToString(levelIdx.Timeframe), " ", prefix, " ERROR Sup Levels differ time0=", iTime(NULL, levelIdx.Timeframe, 0)); }
      ret = false;
   }
   if (!SupDemLevel::CompareLevels("Dems", demLevels, validateDems, !validateAll, LogLevel))
   {
      if (LogLevel >= LogError) { Print(TimeframeToString(levelIdx.Timeframe), " ", prefix, " ERROR Dem Levels differ time0=", iTime(NULL, levelIdx.Timeframe, 0)); }
      ret = false;
   }
   ArrayDelete(validateSups);
   ArrayDelete(validateDems);
   return ret;
}


SupDemLevel*
SupDemControlAdvanced::CreateNewSupply(const SupDemLevelIdx& levelIdx, int barNum)
{
   SupDemLevel* ret = new SupDemLevel();
   
   ret.BarNum = barNum;
   ret.Timestamp = iTime(NULL, levelIdx.Timeframe, barNum);
   ret.PriceHigh = iHigh(NULL, levelIdx.Timeframe, barNum);
   ret.PriceLow = iCustomSd(levelIdx.SupLow, barNum);
   
   if (levelIdx.PopulateScoresAndRetouches())
   {
      ret.Score = (int)iCustomSd(SD_SUPSCORE, barNum);
      ret.IsStrong = (bool)iCustomSd(SD_SUPSTRONG, barNum);
      ret.Retouch = (int)iCustomSd(SD_SUPRETOUCH, barNum);
   }
   return ret;
}


SupDemLevel*
SupDemControlAdvanced::CreateNewDemand(const SupDemLevelIdx& levelIdx, int barNum)
{
   SupDemLevel* ret = new SupDemLevel();
   
   ret.BarNum = barNum;
   ret.Timestamp = iTime(NULL, levelIdx.Timeframe, barNum);
   ret.PriceHigh = iCustomSd(levelIdx.DemHigh, barNum);
   ret.PriceLow = iLow(NULL, levelIdx.Timeframe, barNum);
   
   if (levelIdx.PopulateScoresAndRetouches())
   {
      ret.Score = (int)iCustomSd(SD_DEMSCORE, barNum);
      ret.IsStrong = (bool)iCustomSd(SD_DEMSTRONG, barNum);
      ret.Retouch = (int)iCustomSd(SD_DEMRETOUCH, barNum);
   }
   return ret;
}


void
SupDemControlAdvanced::UpdateRetouchesSupply(SupDemLevel& sup)
{
   int barNum = sup.BarNum;
   
   sup.Score = (int)iCustomSd(SD_SUPSCORE, barNum);
   sup.IsStrong = (bool)iCustomSd(SD_SUPSTRONG, barNum);
   sup.Retouch = (int)iCustomSd(SD_SUPRETOUCH, barNum);
}


void
SupDemControlAdvanced::UpdateRetouchesDemand(SupDemLevel& dem)
{
   int barNum = dem.BarNum;
   
   dem.Score = (int)iCustomSd(SD_DEMSCORE, barNum);
   dem.IsStrong = (bool)iCustomSd(SD_DEMSTRONG, barNum);
   dem.Retouch = (int)iCustomSd(SD_DEMRETOUCH, barNum);
}

#endif