
#ifndef BLAHTECH_SUP_DEM_CONTROLLER_MQH
#define BLAHTECH_SUP_DEM_CONTROLLER_MQH

#property description "Blahtech Supply Demand Controller"
#property link        "http://www.blahtech.co.uk"
#property strict

#include "BlahtechSupDemLevel.mqh"
#include "BlahtechSupDemCustomArgs.mqh"

class SupDemControlBasic
{
public:   
   bool              NewBar;
   
   bool              HasSup;
   int               SupBarNum;
   datetime          SupTimestamp;
   double            SupPriceHigh;
   double            SupPriceLow;
   int               SupScore;
   int               SupIsStrong;
   int               SupRetouch;

   bool              HasDem;   
   int               DemBarNum;
   datetime          DemTimestamp;
   double            DemPriceHigh;
   double            DemPriceLow;
   int               DemScore;
   int               DemIsStrong;
   int               DemRetouch;
      
   int               OverallTrend;
   ENUM_TIMEFRAMES   Timeframe1;
   int               Trend1;
   int               Engulfing1;
   double            QTop1;
   double            QBottom1;
   int               SupDemHint1;
   
   ENUM_TIMEFRAMES   Timeframe2;
   int               Trend2;
   int               Engulfing2;
   double            QTop2;
   double            QBottom2;
   int               SupDemHint2;
   
   ENUM_TIMEFRAMES   Timeframe3;
   int               Trend3;
   int               Engulfing3;
   double            QTop3;
   double            QBottom3;
   
   double            QHi1;
   double            QHi2;
   double            QHi3;
   double            QLow1;
   double            QLow2;
   double            QLow3;
   
private:
   SupDemCustomArgs* CustomArgs;
   LogLevelEnum      LogLevel;
   SdEaValidateEnum  ValidateIncremental;
   SdEaRetouchMode   UpdateRetouches;
   int               IndicatorHandle;
   bool              ValidateActive;
   uint              TickCount;
   datetime          PrevTime1_0;
   
public:
   ~SupDemControlBasic();
   int             Init(SupDemCustomArgs* customArgs, SdEaRetouchMode updateRetouches,  LogLevelEnum logLevel, SdEaValidateEnum  validateInc);
   SdEaUpdateEnum  Populate(datetime time0);
   
private:
   double          iCustomSd(SdBufferIndexEnum index, int shift = 0);
};


void
SupDemControlBasic::~SupDemControlBasic()
{
#ifndef __MQL4__
   if (IndicatorHandle > 0)
   {
      IndicatorRelease(IndicatorHandle);
   }
#endif
}


double
SupDemControlBasic::iCustomSd(SdBufferIndexEnum index, int shift)
{
#ifdef __MQL4__
   return 
      iCustom(
         CustomArgs.StrSymbol, CustomArgs.Timeframe, CustomArgs.IndicatorPath,
         CustomArgs.ExpertMode, CustomArgs.MinBarsLevel, CustomArgs.MaxBars, CustomArgs.ForceRecalcBars, CustomArgs.ParameterValues,
         index, shift);
#else
   double arr[1];
   int    retIdx = CopyBuffer(IndicatorHandle, index, shift, 1, arr);
   return retIdx > 0 ? arr[0] : EMPTY_VALUE;
#endif
}


int
SupDemControlBasic::Init(SupDemCustomArgs* customArgs, SdEaRetouchMode updateRetouches,  LogLevelEnum logLevel, SdEaValidateEnum  validateInc)
{
   int ret = INIT_FAILED;
   
   if (LogLevel >= LogInfo) { Print("SupDemControlBasic::Init - Start"); }
   
   CustomArgs = customArgs;
   UpdateRetouches = updateRetouches;
   LogLevel = logLevel;
   ValidateIncremental = validateInc;
   ValidateActive = ValidateIncremental != SD_EA_VAL_NONE;
   
   Timeframe1 = MappedTimeframeToTimeframe(TIMEFRAME_MAP1, CustomArgs.Timeframe1);
   Timeframe2 = MappedTimeframeToTimeframe(TIMEFRAME_MAP2, CustomArgs.Timeframe2);
   Timeframe3 = MappedTimeframeToTimeframe(TIMEFRAME_MAP3, CustomArgs.Timeframe3);
   
   CustomArgs.ParameterValues += "Timeframe1=" + IntegerToString(Timeframe1) + ";";
   CustomArgs.ParameterValues += "Timeframe2=" + IntegerToString(Timeframe2) + ";";
   CustomArgs.ParameterValues += "Timeframe3=" + IntegerToString(Timeframe3) + ";";
   
   Print("SupDemControlBasic::Init - ParameterValues=");
   Print(CustomArgs.ParameterValues);

#ifdef __MQL4__
   ret = INIT_SUCCEEDED;
#else
   IndicatorHandle =
      iCustom(
         customArgs.StrSymbol, customArgs.Timeframe, customArgs.IndicatorPath,
         customArgs.ExpertMode, customArgs.MinBarsLevel, customArgs.MaxBars, customArgs.ForceRecalcBars, customArgs.ParameterValues);
         
   ret = IndicatorHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
#endif
   if (LogLevel >= LogInfo) { Print("SupDemControlBasic::Init - IndicatorPath=", CustomArgs.IndicatorPath); }
   return ret;
}


SdEaUpdateEnum
SupDemControlBasic::Populate(datetime time0)
{
   SdEaUpdateEnum retChanged = (SdEaUpdateEnum)iCustomSd(SD_CHANGED); // Only do work when values have changed

   TickCount++;
   NewBar = time0 != PrevTime1_0;
   
   if (retChanged > SD_EA_UPDATE_NONE || NewBar)
   {
      if ((retChanged > SD_EA_UPDATE_NONE && LogLevel >= LogDebug) || LogLevel >= LogAll) { Print("SupDemControlBasic::Populate TickCount=", TickCount, " retChanged=", retChanged); }

      // ######################################################################################################
      // ##  For optimal performance - Comment out iCustom calls below for any variables that are not required  
      // ######################################################################################################
      
      SupBarNum    = (int)iCustomSd(SD_SUPBARNUM);       // First Supply
      HasSup       = SupBarNum > 0 && SupBarNum != EMPTY_VALUE;
      SupTimestamp = HasSup ? iTime(NULL, Timeframe1, SupBarNum) : 0;
      SupPriceHigh = HasSup ? iHigh(NULL, Timeframe1, SupBarNum) : 0;
      SupPriceLow  = HasSup ? iCustomSd(SD_SUPLOW, SupBarNum) : 0;
      SupScore     = HasSup ? (int)iCustomSd(SD_SUPSCORE, SupBarNum) : 0;
      SupIsStrong  = HasSup ? (bool)iCustomSd(SD_SUPSTRONG, SupBarNum) : false;
      SupRetouch   = HasSup ? (int)iCustomSd(SD_SUPRETOUCH, SupBarNum) : 0;
      
      DemBarNum    = (int)iCustomSd(SD_DEMBARNUM);       // First Demand
      HasDem       = DemBarNum > 0 && DemBarNum != EMPTY_VALUE;
      DemTimestamp = HasDem ? iTime(NULL, Timeframe1, DemBarNum) : 0;
      DemPriceHigh = HasDem ? iCustomSd(SD_DEMHIGH, DemBarNum) : 0;
      DemPriceLow  = HasDem ? iLow(NULL, Timeframe1, DemBarNum) : 0;
      DemScore     = HasDem ? (int)iCustomSd(SD_DEMSCORE, DemBarNum) : 0;
      DemIsStrong  = HasDem ? (bool)iCustomSd(SD_DEMSTRONG, DemBarNum) : false;
      DemRetouch   = HasDem ? (int)iCustomSd(SD_DEMRETOUCH, DemBarNum) : 0;
      
      OverallTrend = (int)iCustomSd(SD_OVERALLTREND);  // Overall Trend
      Trend1       = (int)iCustomSd(SD_TREND1);        // Timeframe 1
      Engulfing1   = (int)iCustomSd(SD_ENGULFING1);
      QTop1        = iCustomSd(SD_QTOP1);
      QBottom1     = iCustomSd(SD_QBOTTOM1);
      SupDemHint1  = (int)iCustomSd(SD_SUPDEMHINT1);
      Trend2       = (int)iCustomSd(SD_TREND2);        // Timeframe 2
      Engulfing2   = (int)iCustomSd(SD_ENGULFING2);
      QTop2        = iCustomSd(SD_QTOP2);
      QBottom2     = iCustomSd(SD_QBOTTOM2);
      SupDemHint2  = (int)iCustomSd(SD_SUPDEMHINT2);
      Trend3       = (int)iCustomSd(SD_TREND3);        // Timeframe 3
      Engulfing3   = (int)iCustomSd(SD_ENGULFING3);
      QTop3        = iCustomSd(SD_QTOP3);
      QBottom3     = iCustomSd(SD_QBOTTOM3);
      
      // Calculate the QHi and QLow values
      QHi1         = QTop1 * 0.75 + QBottom1 * 0.25;
      QHi2         = QTop2 * 0.75 + QBottom2 * 0.25;
      QHi3         = QTop3 * 0.75 + QBottom3 * 0.25;
      QLow1        = QTop1 * 0.25 + QBottom1 * 0.75;
      QLow2        = QTop2 * 0.25 + QBottom2 * 0.75;
      QLow3        = QTop3 * 0.25 + QBottom3 * 0.75;
      
      PrevTime1_0 = time0;
   }
   return retChanged;
}

#endif
