
#property description "Blahtech Supply Demand Expert Advisor - Advanced Multi"
#property version     "3.2"
#property strict

#include "BlahtechSupDemControlAdvanced.mqh"

#ifdef __MQL4__
input string             IndicatorPath = "Market\\Blahtech Supply Demand";          // Indicator Path
#else
input string             IndicatorPath = "Market\\Blahtech Supply Demand MT5.ex5";  // Indicator Path
#endif
input SdEaModeEnum       ExpertMode = SD_EA_EXPERT_NO_DRAW;                         // Expert Mode
input int                MinBarsLevel = 3;                                          // Min Bars For Level
input int                MaxBars = 20000;                                           // Max Bars
input int                ForceRecalcBars = 10000;                                   // Force Recalc Bars
input string             ParameterValues = "";                                      // Parameter Key Value Pairs
//input string             ParameterValues = "ZoneStyle1=2;InfoLevel=1 S;";         // Parameter Key Value Pairs
input MAPPED_TIMEFRAMES  Timeframe1 = MAP_PERIOD_CURRENT;                           // Timeframe 1
input MAPPED_TIMEFRAMES  Timeframe2 = MAP_PERIOD_MAPPED;                            // Timeframe 2
input MAPPED_TIMEFRAMES  Timeframe3 = MAP_PERIOD_MAPPED;                            // Timeframe 3
input SdEaRetouchMode    UpdateRetouches = SD_EA_RETOUCH_FIRST;                     // Update Retouches Mode
input SdEaValidateEnum   ValidateIncremental = SD_EA_VAL_NONE;                      // Validate Incremental
input bool               ForceFullUpdate = false;                                   // Force Full Update Every Change
input LogLevelEnum       LogLevel = LogError;                                       // Log Level


SupDemCustomArgs*       ExtCustomArgs1;
SupDemControlAdvanced*  ExtControlAdvanced1;
SupDemCustomArgs*       ExtCustomArgs2;
SupDemControlAdvanced*  ExtControlAdvanced2;


int OnInit()
{
   if (LogLevel >= LogInfo) { Print("OnInit - Start"); }
   
   ExtCustomArgs1 = new SupDemCustomArgs();
   ExtControlAdvanced1 = new SupDemControlAdvanced();
   
   ExtCustomArgs1.IndicatorPath = IndicatorPath;
   ExtCustomArgs1.ExpertMode = ExpertMode;
   ExtCustomArgs1.MinBarsLevel = MinBarsLevel;
   ExtCustomArgs1.MaxBars = MaxBars;
   ExtCustomArgs1.ForceRecalcBars = ForceRecalcBars;
   ExtCustomArgs1.Timeframe1 = Timeframe1;
   ExtCustomArgs1.Timeframe2 = Timeframe2;
   ExtCustomArgs1.Timeframe3 = Timeframe3;
   
   // ###################################################
   // ## Uncomment and/or modify to add parameter values
   // ###################################################
   ExtCustomArgs1.ParameterValues = "values:"
   "Id1=1;"
   "Hud=0;"
   "QPoints=0;"
   "SumYOffset=110;"
   "FiltersEnable=1;"
   "FilterMaxLevels=4;"
   "FilterOveralMove1=3.0;"
   "ZoneStyleInvert=1;"
   //"SummaryMode=0;"
   //"TimeframeVisible2=1;"
   //"Timeframe4=-3;"
   //"InfoLevel=8;"
   + ParameterValues; // Append extra values set in input parameter
   
   int ret1 = ExtControlAdvanced1.Init(ExtCustomArgs1, UpdateRetouches, LogLevel, ValidateIncremental, ForceFullUpdate);
   
   

   ExtCustomArgs2 = new SupDemCustomArgs();
   ExtControlAdvanced2 = new SupDemControlAdvanced();
   
   ExtCustomArgs2.IndicatorPath = IndicatorPath;
   ExtCustomArgs2.ExpertMode = ExpertMode;
   ExtCustomArgs2.MinBarsLevel = MinBarsLevel;
   ExtCustomArgs2.MaxBars = MaxBars;
   ExtCustomArgs2.ForceRecalcBars = ForceRecalcBars;
   ExtCustomArgs2.Timeframe1 = Timeframe1;
   ExtCustomArgs2.Timeframe2 = Timeframe2;
   ExtCustomArgs2.Timeframe3 = Timeframe3;
   
   // ###################################################
   // ## Uncomment and/or modify to add parameter values
   // ###################################################
   ExtCustomArgs2.ParameterValues = "values:"
   "Id1=2;"
   "Hud=0;"
   "QPoints=0;"
   "FiltersEnable=1;"
   "FilterMaxLevels=3;"
   "FilterOveralMove1=3.0;"
   //"SummaryMode=0;"
   //"TimeframeVisible2=1;"
   //"ZoneStyle1=2;"
   //"Timeframe4=-3;"
   //"InfoLevel=8;"
   + ParameterValues; // Append extra values set in input parameter
   
   int ret2 = ExtControlAdvanced2.Init(ExtCustomArgs2, UpdateRetouches, LogLevel, ValidateIncremental, ForceFullUpdate);
   
   
   int ret = ret1 == INIT_SUCCEEDED && ret2 == INIT_SUCCEEDED ? INIT_SUCCEEDED : INIT_FAILED;
   
   if (LogLevel >= LogInfo) { Print("OnInit - ret=", ret); }
   return ret;
}


void OnDeinit(const int reason)
{
   DeleteCheck(ExtControlAdvanced2);
   DeleteCheck(ExtCustomArgs2);
   DeleteCheck(ExtControlAdvanced1);
   DeleteCheck(ExtCustomArgs1);
   ChartRedraw();
}


void OnTick()
{
   datetime        time0 = iTime(NULL, 0, 0);                        // ###################################################################################
   SdEaUpdateEnum  sdChanged1 = ExtControlAdvanced1.Populate(time0); // ## For optimal performance - Edit Populate method and comment out unused variables
   SdEaUpdateEnum  sdChanged2 = ExtControlAdvanced2.Populate(time0); // ## For optimal performance - Edit Populate method and comment out unused variables
                                                                     // ###################################################################################

   if (sdChanged1 > SD_EA_UPDATE_NONE || sdChanged2 > SD_EA_UPDATE_NONE || ExtControlAdvanced1.NewBar)
   {
      if (sdChanged1 > SD_EA_UPDATE_NONE)
      {
         if (sdChanged1 == SD_EA_UPDATE_FULL)
         {
            // Add user code for full populate
            
         }
         // Add user code for incremental populate
         
         //if (ArraySize(ExtControlAdvanced1.SupLevels1) > 1)
         //{
         //   SupDemLevel* secondSupLevel = ExtControlAdvanced1.SupLevels1[1];
         //   double       secondSupLevelHigh = secondSupLevel.PriceHigh;
         //}
      }
      
      if (sdChanged2 > SD_EA_UPDATE_NONE)
      {
         if (sdChanged2 == SD_EA_UPDATE_FULL)
         {
            // Add user code for full populate
            
         }
         // Add user code for incremental populate
         
      }
      
      if (ExtControlAdvanced1.NewBar)
      {
         // Add user code for new bar (current timeframe)
         
      }
   }
   // Add user code for each tick
}

