
#ifndef BLAHTECH_UTILS_MQH
#define BLAHTECH_UTILS_MQH

#property description "Blahtech Utils"
#property link        "http://www.blahtech.co.uk"
#property strict

enum SdBufferIndexEnum
{
   SD_EMPTY = 0,
   SD_CHANGED = 1,         // Changed
   SD_SUPBARNUM = 2,       // Supplies
   SD_SUPLOW = 3,
   SD_SUPSCORE = 4,
   SD_SUPSTRONG = 5,
   SD_SUPRETOUCH = 6,
   SD_SUPRESERVED = 7,
   SD_DEMBARNUM = 8,       // Demands
   SD_DEMHIGH = 9,
   SD_DEMSCORE = 10,
   SD_DEMSTRONG = 11,
   SD_DEMRETOUCH = 12,
   SD_DEMRESERVED = 13,
   SD_OVERALLTREND = 14,   // Overall Trend
   SD_TREND1 = 15,         // Timeframe 1
   SD_ENGULFING1 = 16,
   SD_QTOP1 = 17,
   SD_QBOTTOM1 = 18,
   SD_SUPDEMHINT1 = 19,
   SD_TREND2 = 20,         // Timeframe 2
   SD_ENGULFING2 = 21,
   SD_QTOP2 = 22,
   SD_QBOTTOM2 = 23,
   SD_SUPDEMHINT2 = 24,
   SD_TREND3 = 25,         // Timeframe 3
   SD_ENGULFING3 = 26,
   SD_QTOP3 = 27,
   SD_QBOTTOM3 = 28,
   
   SD_SUPBARNUM2 = 29,     // Supplies TF 2
   SD_SUPLOW2 = 30,
   SD_DEMBARNUM2 = 31,     // Demands TF 2
   SD_DEMHIGH2 = 32,
   SD_SUPBARNUM3 = 33,     // Supplies TF 3
   SD_SUPLOW3 = 34,
   SD_DEMBARNUM3 = 35,     // Demands TF 3
   SD_DEMHIGH3 = 36,
};

enum SdEaUpdateEnum
{
   SD_EA_UPDATE_NONE = 0,
   SD_EA_UPDATE_INCREMENTAL = 1,
   SD_EA_UPDATE_FULL = 2,
};

enum SdEaModeEnum
{
   SD_EA_EXPERT = 1,          // EA - Draw           (Slower)
   SD_EA_EXPERT_NO_DRAW = 2,  // EA - No Drawing (Faster)
};

enum SdEaRetouchMode
{
   SD_EA_RETOUCH_FIRST = 0,   // First Level Only (Each Change)
   SD_EA_RETOUCH_ALL = 1,     // All Levels          (Each New Bar)
};

enum SdEaValidateEnum
{
   SD_EA_VAL_NONE = 0,        // None
   SD_EA_VAL_ALL = 1,         // All
   SD_EA_VAL_FIRST = 2,       // First
};


enum LogLevelEnum
{
   LogNone = 0,      // None
   LogError = 1,     // Error
   LogInfo = 2,      // Info
   LogDebug = 3,     // Debug
   LogAll = 4,       // All
};

enum RECTANGLE_STYLE
{
   RECT_STYLE_SOLID = 0,      // Solid
   RECT_STYLE_LINE = 5,       // Line
   RECT_STYLE_DASH = 1,       // Dash
   RECT_STYLE_DOT = 2,        // Dotted
   RECT_STYLE_DASHDOT = 3,    // Dash-Dotted
   RECT_STYLE_DASHDOTDOT = 4  // Dash-Dot-Dotted
};

enum MAPPED_TIMEFRAMES
{
   MAP_PERIOD_CURRENT = 0,   // Current
   MAP_PERIOD_MAPPED = -2,   // Mapped
   MAP_PERIOD_DISABLED = -3, // Disabled
   MAP_PERIOD_M1 = 1,        // 1 Minute
   MAP_PERIOD_M5 = 5,        // 5 Minute
   MAP_PERIOD_M15 = 15,      // 15 Minutes
   MAP_PERIOD_M30 = 30,      // 30 Minutes
   MAP_PERIOD_H1 = 60,       // 1 Hour
   MAP_PERIOD_H4 = 240,      // 4 Hours
   MAP_PERIOD_D1 = 1440,     // 1 Day
   MAP_PERIOD_W1 = 10080,    // 1 Week
   MAP_PERIOD_MN1 = 43200,   // 1 Month
   MAP_PERIOD_Q1 = 129600,   // 1 Quarter
   MAP_PERIOD_Y1 = 518400,   // 1 Year
};


bool StringStartsWith(string str, string s)
{
   return StringFind(str, s) == 0;
}


string TimeframeToString(int timeframe = PERIOD_CURRENT)
{
   string ret;
   
   if (timeframe == PERIOD_CURRENT)
   {
      timeframe = _Period;
   }
   switch(timeframe)
   {
      case PERIOD_M1:  ret="M1";  break;
      case PERIOD_M5:  ret="M5";  break;
      case PERIOD_M15: ret="M15"; break;
      case PERIOD_M30: ret="M30"; break;
      case PERIOD_H1:  ret="H1";  break;
      case PERIOD_H4:  ret="H4";  break;
      case PERIOD_D1:  ret="D1";  break;
      case PERIOD_W1:  ret="W1";  break;
      case PERIOD_MN1: ret="MN1";  break;
      case 129600: ret="Q1";  break;
      case 518400: ret="Y1";  break;
      default: ret = "UNKNOWN";   break;
   }
   return ret;
}


ENUM_TIMEFRAMES StringToTimeframe(string str)
{
   ENUM_TIMEFRAMES ret = PERIOD_CURRENT;
   
   if      (str == "M1")   ret = PERIOD_M1;
   else if (str == "M5")   ret = PERIOD_M5;
   else if (str == "M15")  ret = PERIOD_M15;
   else if (str == "M30")  ret = PERIOD_M30;
   else if (str == "H1")   ret = PERIOD_H1;
   else if (str == "H4")   ret = PERIOD_H4;
   else if (str == "D1")   ret = PERIOD_D1;
   else if (str == "W1")   ret = PERIOD_W1;
   else if (str == "MN1")  ret = PERIOD_MN1;
   else if (str == "Q1")   ret = (ENUM_TIMEFRAMES)129600;
   else if (str == "Y1")   ret = (ENUM_TIMEFRAMES)518400;
   else if (str == "NONE") ret = PERIOD_CURRENT;
   
   return ret;
}


ENUM_TIMEFRAMES MappedTimeframeToTimeframe(string map, MAPPED_TIMEFRAMES mapTimeframe = MAP_PERIOD_MAPPED)
{
   ENUM_TIMEFRAMES ret = (ENUM_TIMEFRAMES)_Period;
   
   switch(mapTimeframe)
   {
      case MAP_PERIOD_CURRENT:
         break;
      case MAP_PERIOD_DISABLED:
         ret = PERIOD_CURRENT;
         break;
      case MAP_PERIOD_MAPPED:
      {
         string  currPeriodStr = TimeframeToString(ret);
         string  mapPairs[];
      
         StringSplit(map, ',', mapPairs);
      
         for(int m = 0; m < ArraySize(mapPairs); m++)
         {
            if (StringStartsWith(mapPairs[m], currPeriodStr + "="))
            {
               string retStr = StringSubstr(mapPairs[m], StringLen(currPeriodStr) + 1);
               ret = StringToTimeframe(retStr);
            }
         }
         break;
      }
      case MAP_PERIOD_M1:  ret = PERIOD_M1;  break;
      case MAP_PERIOD_M5:  ret = PERIOD_M5;  break;
      case MAP_PERIOD_M15: ret = PERIOD_M15; break;
      case MAP_PERIOD_M30: ret = PERIOD_M30; break;
      case MAP_PERIOD_H1:  ret = PERIOD_H1;  break;
      case MAP_PERIOD_H4:  ret = PERIOD_H4;  break;
      case MAP_PERIOD_D1:  ret = PERIOD_D1;  break;
      case MAP_PERIOD_W1:  ret = PERIOD_W1;  break;
      case MAP_PERIOD_MN1: ret = PERIOD_MN1; break;
      default:
         ret = (ENUM_TIMEFRAMES)mapTimeframe;
         break;
   }
   return ret;
}

template <typename T>
void DeleteCheck(T* obj)
{
   if (CheckPointer(obj) == POINTER_DYNAMIC)
   {
      delete obj;
   }
}

template <typename T>
T ArrayAppend(T &array[], T newObj, int reserveSize = 1000)
{
   int size = ArraySize(array);
   ArrayResize(array, size+1, reserveSize);
   array[size] = newObj;
   return newObj;
}


template <typename T>
T ArrayPrepend(T &array[], T newObj, int reserveSize = 1000)
{
   int size = ArraySize(array);
   ArrayResize(array, size+1, reserveSize);
   
   for (int i = size; i >= 1; i--)
   {
      array[i] = array[i-1];
   }
   return array[0] = newObj;
}


template <typename T>
void ArrayDelete(T &array[], int reserveSize = 1000)
{
   int size = ArraySize(array);
   
   if (size > 0)
   {
      for (int i = 0; i < size; i++)
      {
         delete array[i];
      }
      ArrayResize(array, 0, reserveSize);
   }
}


template <typename T>
void ArrayDeleteFromStart(T &array[], int num, int reserveSize = 1000)
{
   if (num > 0)
   {
      int size = ArraySize(array);
      
      if (num > size)
      {
         num = size;
      }
      for (int i = 0; i < num; i++)
      {
         delete array[i];
      }
      int newSize = size - num;
      
      for (int i = 0; i < newSize; i++)
      {
         array[i] = array[num++];
      }
      ArrayResize(array, newSize, reserveSize);
   }
}


template <typename T>
void ArrayDeleteFromEnd(T &array[], int num, int reserveSize = 1000)
{
   if (num > 0)
   {
      int size = ArraySize(array);
      
      if (num > size)
      {
         num = size;
      }
      for (int i = size - num; i < size; i++)
      {
         delete array[i];
      }
      ArrayResize(array, size - num, reserveSize);
   }
}


template <typename T>
string ArrayToString(T &array[], int first = 0, int max = 0)
{
   string ret = "";
   int    size = max > 0 ? max : ArraySize(array);
   
   for (int i = first; i < size; i++)
   {
      ret += (i > first ? " " : "") + (string)array[i];
   }
   return ret;
}


template <typename T>
string ArrayBarNumToString(T &array[], int first = 0, int max = 0)
{
   string ret = "";
   int    size = max > 0 ? max : ArraySize(array);
   
   for (int i = first; i < size; i++)
   {
      ret += (i > first ? " " : "") + (string)array[i].BarNum;
   }
   return ret;
}

#endif
